﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Main : MonoBehaviour {

	public Unit prefabUnit;
	public Unit prefabUnitEnemy;
	public Transform prefabBlood;
	
	public AudioClip soundBlip;
	public AudioClip soundDeath;
	
	public GUISkin theSkin;

	public Transform moveTarget;

	public Transform prefabSceneryTree;


	private Vector3 unitStartPos = new Vector3 (-7.5f, 0f, 0f);
	private Vector3 unitEnemyStartPos = new Vector3 (7.5f, 0f, 0f);

	private Vector2 sceneBoundTop = new Vector2 (-7, 4);
	private Vector2 sceneBoundBottom = new Vector2 (7, -4);


	private int spawnCount = 5;
	private int sceneTreeCount = 6;

	private List<Unit> units;
	private List<Unit> unitsEnemy;
	private List<Trench> trenches;

	private Vector3 lastClick = Vector3.zero;
	
	private int score = 0;

	// Use this for initialization
	void Start () {

		units = new List<Unit> ();
		unitsEnemy = new List<Unit> ();

		trenches = new List<Trench> ();

		for (int i = 0; i < spawnCount; i++) {
			Unit thisUnit = Instantiate(prefabUnit, unitStartPos, Quaternion.identity) as Unit;
			units.Add(thisUnit);
		}

		for (int i = 0; i < spawnCount; i++) {
			Unit thisUnit = Instantiate(prefabUnitEnemy, unitEnemyStartPos, Quaternion.identity) as Unit;
			unitsEnemy.Add(thisUnit);
		}

		setupTrenches ();

		StartCoroutine("setupScenery");

		StartCoroutine ("enemyPathing");
		StartCoroutine ("winCheck");

		lastClick = unitStartPos;
		// moveTarget.position = unitStartPos;
		playBlip ();
	}
	
	// Update is called once per frame
	void Update () {
		processClick ();

		moveTarget.Rotate (Vector3.forward * 60f * Time.deltaTime);
		if (moveTarget.position != lastClick) {
			moveTarget.position = Vector3.MoveTowards(moveTarget.position, lastClick, 25f * Time.deltaTime);
		}
	}
	
	
	void OnGUI() {
		
		GUI.skin = theSkin;
		
		GUI.skin.label.fontSize = GUI.skin.box.fontSize = GUI.skin.button.fontSize = 12;
		
		GUI.Box(new Rect(10,10, 120, 100), "Trench War!\nby\n ImDeity");
		
		if (GUI.Button(new Rect(20, 80, 100, 24), "Restart")) {
			Debug.LogWarning("Restart");
			Application.LoadLevel("Title");
		}
		
		GUI.Box(new Rect(Screen.width - 140, 10, 120, 40), "Score: " + score + "");
		
		
	}
	


	public void createBloodAt(Vector3 inPos) {
		inPos.z = 0.5f;
		Instantiate (prefabBlood, inPos, Quaternion.identity);
	}


	IEnumerator enemyPathing() {

		while (true) {
			yield return new WaitForSeconds(5f);

			foreach (Unit thisUnit in unitsEnemy) {
				Vector3 targetLoc = findNextTarget(thisUnit);
				if (targetLoc != Vector3.zero) {
					thisUnit.moveTo(targetLoc);
					yield return null;
				}
			}
		}
	}


	IEnumerator winCheck() {
		while (true) {
			yield return new WaitForSeconds(5f);

			int enemyAlive = 0;
			foreach (Unit thisUnit in unitsEnemy) {
				if (thisUnit.health > 0) {
					enemyAlive++;
				}
			}
			
			int unitAlive = 0;
			foreach (Unit thisUnit in units) {
				if (thisUnit.health > 0) {
					unitAlive++;
				}
			}
			
			if (enemyAlive == 0) { 
			
				saveStats(enemyAlive, unitAlive);
				playBlip ();
				Application.LoadLevel ("Outro");
				break;
			} else if (unitAlive == 0) { 
			
				saveStats(enemyAlive, unitAlive);
				playBlip ();
				Application.LoadLevel ("Outro");
				break;
			}

		}

	}
	
	
	private void saveStats(int enemyAlive, int unitAlive) {
		
		PlayerPrefs.SetInt("enemyAlive", enemyAlive);
		PlayerPrefs.SetInt("unitAlive", unitAlive);
		PlayerPrefs.SetInt("lastScore", score);
		
		PlayerPrefs.Save();
	
	}
	
	
	public void scoreIncr(int inAmt) {
	
		score += inAmt;
		
	}


	Vector3 findNextTarget(Unit thisUnit) {
		Vector3 closestTarget = Vector3.zero;
		float closestDist = 100000f;

		foreach (Trench trench in trenches) {
			if (thisUnit == null) { continue; }
			float thisDist = Vector3.Distance(trench.transform.position, thisUnit.transform.position);
			if (thisDist > 2f && thisDist < closestDist) {
				closestDist = thisDist;
				closestTarget = trench.transform.position;
			}

		}
		return closestTarget;
	}


	IEnumerator setupScenery() {
		for (int i = 0; i < sceneTreeCount; i++) {
			
			Transform thisTree = Instantiate(prefabSceneryTree) as Transform;
			thisTree.position = new Vector3(sceneBoundBottom.x * 2, sceneBoundBottom.y * 2, 0);

			Vector3 newLoc = Vector3.zero;
			while (true) {
				newLoc = new Vector3(
					Random.Range(sceneBoundTop.x, sceneBoundBottom.x), 
					Random.Range(sceneBoundTop.y, sceneBoundBottom.y),
					0f);
				bool safeLoc = true;
				foreach (Trench trench in trenches) {
					if (Vector3.Distance(trench.transform.position, newLoc) < 3f) {
						safeLoc = false;
						break;
					}
				}

				if (safeLoc) {
					thisTree.position = newLoc;
					break;
				}
				yield return null;
			}
			yield return null;
		}
		Debug.Log ("* Scenery setup complete");
	}

	void setupTrenches() {

		GameObject[] foundTrenches = GameObject.FindGameObjectsWithTag ("Trenches");
		foreach (GameObject thisTrench in foundTrenches) {
			trenches.Add(thisTrench.GetComponent<Trench>());
		}
		Debug.Log ("* Added " + trenches.Count + " Trenches");
	}


	void processClick() {

		if (!Input.GetMouseButtonUp (0)) { return; }

		Ray ray = Camera.main.ScreenPointToRay (Input.mousePosition);
		RaycastHit hit = new RaycastHit ();
		if (Physics.Raycast (ray, out hit, 100)) {

			Vector3 hitPoint = hit.point;
			hitPoint.z = 0f;

			foreach(Unit thisUnit in units) {
				thisUnit.moveTo(hitPoint);

				lastClick = hitPoint;
				lastClick.z = 0.6f;

			}

		}

	}
	
	public void playBlip(){
		audio.PlayOneShot(soundBlip);
	}
	
	public void playDeath(){
		audio.PlayOneShot(soundDeath, .2f);
	}

}
