﻿using UnityEngine;
using System.Collections;

public class Unit : MonoBehaviour {

	public Transform prefabBullet;
	public Transform prefabHitbar;
	
	private float speed = 1.1f;
	private float maxHealth = 30f;
	public float health = 30f;

	private Transform hitBar;

	private Vector3 lastTarget = Vector3.zero;
	private bool firing = false;
	private bool inTrench = false;

	private bool isEnemy = false;
	private Main main;

	private SpriteRenderer sprRenderer;

	// Use this for initialization
	void Start () {

		main = Camera.main.GetComponent<Main> ();

		lastTarget = transform.position;
		StartCoroutine ("attackLoop");

		if (gameObject.CompareTag ("UnitsEnemy")) {
			isEnemy = true;
		}

		hitBar = Instantiate (prefabHitbar) as Transform;
		sprRenderer = GetComponent<SpriteRenderer> ();
	}
	
	// Update is called once per frame
	void Update () {

		updateHealth ();
	}

	void FixedUpdate() {

		if (!firing && Vector3.Distance (lastTarget, transform.position) > .5f) {

			turnToFace(lastTarget);

			float currentSpeed = speed;
			if (!inTrench) { currentSpeed = speed * .4f; }

			transform.position = Vector3.MoveTowards(transform.position, lastTarget, Time.deltaTime * currentSpeed);
		}

	}


	public void takeDamage(float inDamage) {

		if (inTrench) { inDamage = inDamage * 0.6f; }

		health -= inDamage;
		StartCoroutine("doDamageEffect");

		if (health < 0) { health = 0; }
	}


	private void updateHealth() {
		hitBar.transform.position = transform.position + new Vector3 (0, 0.22f, 0f);
		if (health >= maxHealth) {
			health = maxHealth;
			hitBar.renderer.enabled = false;
		} else {
			if (health <= 0f) {
				killSelf();
				return;
			}
			hitBar.renderer.enabled = true;
			Vector3 newHitscale = Vector3.one;
			newHitscale.x = health / maxHealth;
			hitBar.localScale = newHitscale;
		}
	}


	public void moveTo(Vector3 targetPos) {
		lastTarget = targetPos;
	}

	
	private void turnToFace(Vector3 theTarget) {
		Vector3 targetDir = theTarget - transform.position;
		
		float angle = Mathf.Atan2(targetDir.y, targetDir.x) * Mathf.Rad2Deg;
		transform.rotation = Quaternion.AngleAxis(angle, Vector3.forward);
	}


	void OnTriggerEnter2D(Collider2D collider) {
		if (collider.gameObject.CompareTag ("Trenches")) {
			inTrench = true;
		}
	}


	void OnTriggerExit2D(Collider2D collider) {
		if (collider.gameObject.CompareTag ("Trenches")) {
			inTrench = false;
		}
	}

	IEnumerator attackLoop() {

		// Can always fire from trench
		while (true) {

			// Wait for next fire time
			yield return new WaitForSeconds(Random.Range (1.6f, 2.2f));

			string collideTag = "UnitsEnemy";
			if (isEnemy) { collideTag = "Units"; }

			Transform targetUnit = null;
			while (true) {
				// Range boost in trench
				float attackRange = 3f;
				if (inTrench) { attackRange = 5f; }

				// Find target
				Collider2D[] theTargets = Physics2D.OverlapCircleAll(transform.position, attackRange);
				foreach ( Collider2D thisTarget in theTargets) {
					if (thisTarget.tag == collideTag) {
						// Debug.Log (gameObject.tag + " found enemy " + thisTarget.tag);
						targetUnit = thisTarget.transform;
						break; // out of foreach
					}
				}
				if (targetUnit != null) { break; } // out of while
				yield return new WaitForSeconds(0.5f);
			}

			// Found one, fire
			firing = true;
			yield return new WaitForSeconds(0.4f);
			
			Transform thisBullet = Instantiate (prefabBullet, transform.position + (transform.right * .2f), Quaternion.identity) as Transform;

			// Orient unit/bullet
			if (targetUnit == null) { 
				firing = false;
				continue;
			}
			turnToFace(targetUnit.position);
			thisBullet.rotation = transform.rotation;

			thisBullet.GetComponent<Projectile>().setEnemy(isEnemy);
			
			thisBullet.rigidbody2D.AddForce (transform.right * 100f);
			audio.Play ();
			yield return new WaitForSeconds(.2f);
			firing = false;
		}
	}

	IEnumerator doDamageEffect() {
		sprRenderer.color = Color.yellow;
		yield return new WaitForSeconds (0.3f);
		sprRenderer.color = Color.white;
	}

	public void killSelf() {
	
		if (isEnemy) { main.scoreIncr(10); }
		
		Destroy(hitBar.gameObject);
		
		main.playDeath();
		main.createBloodAt(transform.position);
		
		Destroy(gameObject);
	}


}
